# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
"""Assets module"""

import superdesk
import bson.errors

from flask import request, current_app as app
from content_api.errors import FileNotFoundError
from superdesk import get_resource_service
from superdesk.upload import upload_url as _upload_url
from superdesk.storage.superdesk_file import generate_response_for_file

bp = superdesk.Blueprint("assets", __name__)


@bp.route("/assets/<path:media_id>", methods=["GET"])
def get_media_streamed(media_id):
    if not app.auth.authorized([], "assets", "GET"):
        return app.auth.authenticate()
    try:
        media_id = media_id.split(".")[0]
        media_file = app.media.get(media_id, "upload")
    except bson.errors.InvalidId:
        media_file = None
    if media_file:
        get_resource_service("api_audit").audit_item({"type": "asset", "uri": request.url}, media_id)
        return generate_response_for_file(
            media_file, cache_for=3600 * 24 * 7, content_disposition="inline"  # 7 days cache
        )
    raise FileNotFoundError("File not found on media storage.")


def upload_url(media_id):
    return _upload_url(media_id, view="assets.get_media_streamed")


def init_app(app) -> None:
    superdesk.blueprint(bp, app)
    app.upload_url = upload_url



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import superdesk

from superdesk.services import BaseService
from superdesk.utils import ListCursor


logger = logging.getLogger(__name__)


def view_language(item):
    language = item.copy()
    language["_id"] = language["qcode"]
    language["label"] = language["name"]
    language["language"] = language["qcode"]

    # allow translations
    language.setdefault("source", True)
    language.setdefault("destination", True)

    return language


class LanguagesService(BaseService):
    def get(self, req, lookup):
        """
        Return the list of languages defined on config file.
        """
        languages = superdesk.get_resource_service("vocabularies").get_languages()
        return ListCursor([view_language(lang) for lang in languages])



import superdesk
from eve.flaskapp import Eve
from .assets import assets_bp, unlock_asset_by_user
from .storage_destinations import destinations_bp
from .sets import sets_bp
from superdesk.auth.decorator import blueprint_auth
from flask_babel import _
from .client import get_sams_client


def init_app(app: Eve):
    client = get_sams_client(app)

    app.on_session_end -= unlock_assets_on_logout
    app.on_session_end += unlock_assets_on_logout

    @assets_bp.before_request
    @destinations_bp.before_request
    @sets_bp.before_request
    @blueprint_auth()
    def before_request():
        """
        Add authentication before request to all blueprint
        """
        pass

    @assets_bp.after_request
    @destinations_bp.after_request
    @sets_bp.after_request
    def after_request(response):
        response.headers.set("Access-Control-Allow-Origin", "*")
        response.headers.set("Access-Control-Allow-Headers", "*")
        response.headers.set("Access-Control-Allow-Methods", "*")
        return response

    superdesk.blueprint(destinations_bp, app, client=client)
    superdesk.blueprint(sets_bp, app, client=client)
    superdesk.blueprint(assets_bp, app, client=client)

    superdesk.privilege(
        name="sams", label=_("Sams"), description=_("Access to the SAMS management page (and to upload assets etc)")
    )

    superdesk.privilege(name="sams_manage", label=_("Sams manage"), description=_("Allows management of SAMS Sets etc"))

    superdesk.privilege(
        name="sams_manage_assets", label=_("SAMS Manage Assets"), description=_("Allows management of SAMS Assets")
    )


def unlock_assets_on_logout(user_id, session_id):
    unlock_asset_by_user(user_id, session_id)



from nose.plugins.attrib import attr


def wip(f):
    """Allows to run a single test using a @wip decorator

    i.e.
    from superdesk.tests import TestCase
    from tests import wip


    class TestWIP(TestCase):
        @wip
        def test_something(self):
            self.assertEqual(1, 1)

    then run with:
    nosetests -a wip
    """

    return attr("wip")(f)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""The Sets API to retrieve, create, update, delete the list of ``Sets``.


=====================   =================================================
**endpoint name**        'sets'
**resource title**       'Sets'
**resource url**         [GET] '/sams/sets'
**item url**             [GET] '/sams/sets/<:class:`str`>'
**schema**               :attr:`sams_client.schemas.SET_SCHEMA`
=====================   =================================================
"""

import logging
import superdesk
from flask import request, current_app as app
from superdesk.errors import SuperdeskApiError
from superdesk.notification import push_notification
from apps.auth import get_auth, get_user_id
from .client import get_sams_client

logger = logging.getLogger(__name__)
sets_bp = superdesk.Blueprint("sams_sets", __name__)


@sets_bp.route("/sams/sets", methods=["GET"])
def get():
    """
    Returns a list of all the registered sets
    """
    sets = get_sams_client().sets.search()
    return sets.json(), sets.status_code


@sets_bp.route("/sams/sets/<item_id>", methods=["GET"])
def find_one(item_id):
    """
    Uses item_id and returns the corresponding
    set
    """
    item = get_sams_client().sets.get_by_id(item_id=item_id)
    return item.json(), item.status_code


@sets_bp.route("/sams/sets", methods=["POST"])
def create():
    """
    Creates new sets
    """
    docs = request.get_json()
    post_response = get_sams_client().sets.create(docs=docs, external_user_id=get_user_id(True))
    if post_response.status_code == 201:
        push_notification(
            "sams:set:created",
            item_id=post_response.json()["_id"],
            user_id=get_user_id(True),
            session_id=get_auth()["_id"],
            _etag=post_response.json()["_etag"],
            extension="sams",
        )
    return post_response.json(), post_response.status_code


@sets_bp.route("/sams/sets/<item_id>", methods=["DELETE"])
def delete(item_id):
    """
    Uses item_id and deletes the corresponding set
    """
    try:
        etag = request.headers["If-Match"]
    except KeyError:
        raise SuperdeskApiError.badRequestError("If-Match field missing in header")

    delete_response = get_sams_client().sets.delete(item_id=item_id, headers={"If-Match": etag})
    if delete_response.status_code != 204:
        return delete_response.json(), delete_response.status_code
    if delete_response.status_code == 204:
        remove_set_restriction_from_desks(item_id)
        push_notification(
            "sams:set:deleted",
            item_id=item_id,
            user_id=get_user_id(True),
            session_id=get_auth()["_id"],
            extension="sams",
        )
    return "", delete_response.status_code


def remove_set_restriction_from_desks(set_id_to_remove: str):
    """
    Removes the provided Set from "sams_settings.sets" from all Desks
    """

    desk_service = superdesk.get_resource_service("desks")
    desks_db = app.data.mongo.pymongo("desks").db["desks"]

    # Use pymongo directly as this query doesn't work through services
    for desk in desks_db.find({"sams_settings.allowed_sets": set_id_to_remove}):
        desk["sams_settings"]["allowed_sets"] = [
            set_id for set_id in desk["sams_settings"]["allowed_sets"] if set_id != set_id_to_remove
        ]
        desk_service.patch(id=desk["_id"], updates={"sams_settings": desk["sams_settings"]})


@sets_bp.route("/sams/sets/<item_id>", methods=["PATCH"])
def update(item_id):
    """
    Uses item_id and updates the corresponding set
    """
    try:
        etag = request.headers["If-Match"]
    except KeyError:
        raise SuperdeskApiError.badRequestError("If-Match field missing in header")

    updates = request.get_json()
    update_response = get_sams_client().sets.update(item_id=item_id, updates=updates, headers={"If-Match": etag})
    if update_response.status_code == 200:
        push_notification(
            "sams:set:updated",
            item_id=update_response.json()["_id"],
            user_id=get_user_id(True),
            session_id=get_auth()["_id"],
            _etag=update_response.json()["_etag"],
            extension="sams",
        )
    return update_response.json(), update_response.status_code



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : rails
# Creation: 2021-03-05 13:23

from superdesk import get_resource_service
from superdesk.commands.data_updates import BaseDataUpdate
from eve.utils import config


class DataUpdate(BaseDataUpdate):

    resource = "contacts"

    def forwards(self, mongodb_collection, mongodb_database):
        regions = get_resource_service("vocabularies").find_one(req=None, _id="regions")
        for document in mongodb_collection.find({"contact_state": {"$exists": True}}):
            if document.get("contact_state") and regions:
                region = [t for t in regions.get("items") if t.get("name") == document["contact_state"]]
                if region:
                    contact_state = {
                        "contact_state": {
                            "name": region[0].get("name"),
                            "qcode": region[0].get("qcode"),
                            "translations": region[0].get("translations"),
                        }
                    }
                else:
                    contact_state = {
                        "contact_state": {
                            "name": document["contact_state"],
                            "qcode": document["contact_state"],
                        }
                    }
                mongodb_collection.update(
                    {"_id": document.get(config.ID_FIELD)},
                    {"$set": contact_state},
                )

    def backwards(self, mongodb_collection, mongodb_database):
        raise NotImplementedError()



"""Item Usage Metrics

Usage Metrics is endpoint used by client to store the metrics on user actions like open/preview.
It gets stored into its own collection with details like user and date, plus on the item
as numbers to be able to filter based on such actions::

    {"metrics": {
        "open": 2,
        "preview": 10
    }}

"""

import superdesk

from superdesk.resource import MongoIndexes


class UsageMetricsResource(superdesk.Resource):
    no_privileges = True

    schema = {
        "action": {"type": "string", "required": True},
        "user": superdesk.Resource.rel("users", required=True),
        "item": {"type": "string", "required": True},
        "date": {"type": "datetime"},
    }

    mongo_indexes: MongoIndexes = {
        "item_1": ([("item", 1)], {}),
    }

    url = "usage-metrics"
    item_methods = []
    notifications = False
    resource_methods = ["POST"]


class UsageMetricsService(superdesk.Service):
    def on_created(self, docs):
        for doc in docs:
            archive_service = superdesk.get_resource_service("archive")
            item = archive_service.find_one(req=None, _id=doc["item"])
            archive_service.system_update(
                item["_id"],
                {"$inc": {f"metrics.{doc['action']}": 1}},
                item,
                change_request=True,
                push_notification=False,
            )
        return super().on_created(docs)


def init_app(_app) -> None:
    superdesk.register_resource("usage_metrics", UsageMetricsResource, UsageMetricsService, _app=_app)



from flask_babel import lazy_gettext
import superdesk


superdesk.privilege(
    name="feature_preview",
    label=lazy_gettext("Feature Preview"),
    description=lazy_gettext("Let user toggle Feature Preview on/off."),
)



#!/usr/bin/env python3
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""This module contains tools to manage content for Superdesk editor"""

import re
import uuid
import logging
import lxml.etree as etree
import lxml.html as lxml_html

import superdesk

from textwrap import dedent
from collections.abc import MutableSequence

from flask import current_app as app

from draftjs_exporter.html import HTML
from draftjs_exporter.constants import ENTITY_TYPES, INLINE_STYLES, BLOCK_TYPES
from draftjs_exporter.defaults import STYLE_MAP, BLOCK_MAP
from draftjs_exporter.dom import DOM
from .etree import parse_html, to_string


logger = logging.getLogger(__name__)
DUMMY_RE = re.compile(r"</?dummy_tag>")

ANNOTATION = "ANNOTATION"
MEDIA = "MEDIA"
TABLE = "TABLE"

EDITOR_STATE = "draftjsState"
ENTITY_MAP = "entityMap"
ENTITY_RANGES = "entityRanges"
INLINE_STYLE_RANGES = "inlineStyleRanges"

TAG_STYLE_MAP = {
    "i": INLINE_STYLES.ITALIC,
    "em": INLINE_STYLES.ITALIC,
    "b": INLINE_STYLES.BOLD,
    "strong": INLINE_STYLES.BOLD,
}

TAG_ENTITY_MAP = {
    "a": ENTITY_TYPES.LINK,
}

# FIXME: this is a temporary flag to check that HTML generated by ``generate_fields`` is the
#   same as the one which was already presents if it exists (i.e. the one generated by client).
#   If they differ, a warning will be logged. This is a safety measure to check backend
#   generator before switching off the one from client. More details at SDESK-4911
CHECK_GENERATE_CONSISTENCY = True

# FIXME: those fields are currently hardcoded because they were hardcoded in client too
#   (see https://github.com/superdesk/superdesk-core/pull/1865#issuecomment-632103773).
#   A cleaner way to get field content type should be done (cf. https://dev.sourcefabric.org/browse/SDESK-5316)
TEXT_FIELDS = ["headline", "slugline"]


def get_field_content_state(item, field):
    try:
        return item["fields_meta"][field][EDITOR_STATE][0]
    except (KeyError, AttributeError):
        return None


def get_content_state_fields(item):
    """Return all fields which have a content state"""
    return [field for field in item.get("fields_meta") or {}]


def set_field_content_state(item, field, content_state):
    item.setdefault("fields_meta", {}).update({field: {EDITOR_STATE: [content_state]}})


def get_field_path(item, field):
    keys = field.split(">")
    if len(keys) == 2:
        return item.setdefault("extra", {}), keys[1]
    return item, field


def get_field_value(item, field):
    path, key = get_field_path(item, field)
    return path.get(key)


def set_field_value(item, field, value):
    path, key = get_field_path(item, field)
    path[key] = value


class Entity:
    """Abstraction of a DraftJS entity"""

    def __init__(self, ranges, data):
        self.ranges = ranges
        self.data = data

    @property
    def key(self):
        return str(self.ranges["key"])

    @property
    def type(self):
        return self.data["type"]

    @property
    def is_mutable(self):
        return self.data["mutability"] == "MUTABLE"

    @property
    def content(self):
        return self.data["data"]

    def __str__(self):
        return str(self.data)

    def __repr__(self):
        return "{mutable}{type} entity (key: {key})".format(
            mutable="mutable " if self.is_mutable else "",
            type=self.type,
            key=self.key,
        )


class EntitySequence(MutableSequence):
    def __init__(self, editor, block):
        self._ranges = block.data.setdefault("entityRanges", [])
        self._mapping = editor.content_state.setdefault("entityMap", {})

    def __getitem__(self, key):
        ranges = self._ranges[key]
        data = self._mapping[str(ranges["key"])]
        return Entity(ranges, data)

    def __setitem__(self, key, value):
        if not isinstance(value, Entity):
            raise TypeError("an Entity instance is expected")
        self._ranges[key] = value.ranges
        self._mapping[value.key] = value.data

    def __delitem__(self, key):
        entity_key = str(self._ranges[key]["key"])
        del self._ranges[key]
        del self._mapping[entity_key]

    def __len__(self):
        return len(self._ranges)

    def insert(self, index, value):
        if not isinstance(value, Entity):
            raise TypeError("an Entity instance is expected")
        self._ranges.insert(index, value.ranges)
        self._mapping[value.key] = value.data


class Block:
    """Abstraction of DraftJS block"""

    def __init__(self, editor, data=None, text=None):
        if data is None:
            data = {
                "key": str(uuid.uuid4()),
                "text": text or "",
                "type": "unstyled",
                "depth": 0,
                "inlineStyleRanges": [],
                "entityRanges": [],
                "data": {},
            }
        self.data = data
        self.entities = EntitySequence(editor, self)

    @property
    def depth(self):
        return self.data["depth"]

    @property
    def type(self):
        return self.data["type"]

    @property
    def text(self):
        return self.data.get("text")

    @property
    def key(self):
        return self.data.get("key")

    def __str__(self):
        return self.text

    def __repr__(self):
        return "{type} block with text {text!r} and {len_entities} entities".format(
            type=self.type, text=self.text, len_entities=len(self.entities)
        )


class EmbedBlock(Block):
    def __init__(self, editor, html):
        """
        :param str html: raw HTML to embed
        """
        super().__init__(editor)
        self.data.update(
            {
                "text": " ",
                "type": "atomic",
            }
        )
        entity_data = {
            "type": "EMBED",
            "mutability": "MUTABLE",
            "data": {
                "data": {
                    "html": html,
                }
            },
        }
        entity = Entity(
            ranges={"offset": 0, "length": 1, "key": editor.get_next_entity_key()},
            data=entity_data,
        )
        self.entities.append(entity)


class UnstyledBlock(Block):
    def __init__(self, editor, text=None):
        super().__init__(editor)
        if text:
            self.data.update({"text": text})


class BlockSequence(MutableSequence):
    def __init__(self, editor):
        self.editor = editor
        self._blocks = editor.content_state.setdefault("blocks", {})

    def __getitem__(self, key):
        return Block(self.editor, self._blocks[key])

    def __setitem__(self, key, value):
        if not isinstance(value, Block):
            raise TypeError("a Block instance is expected")
        self._blocks[key] = value.data

    def __delitem__(self, key):
        del self._blocks[key]

    def __len__(self):
        return len(self._blocks)

    def insert(self, index, value):
        if not isinstance(value, Block):
            raise TypeError("a Block instance is expected")
        self._blocks.insert(index, value.data)


class EditorContent:
    """Base class to manage content from editors"""

    def __init__(self):
        raise RuntimeError("This class must not be instantiated directly, use create()")

    @staticmethod
    def create(item, field):
        """Factory for EditorContent"""
        return Editor3Content(item, field)


class DraftJSHTMLExporter:
    def __init__(self, content_editor):
        self.content_editor = content_editor
        # XXX: we need one exporter per DraftJSHTMLExporter because
        #      self.render_media needs to access "entityMap" from local
        #      instance. If MEDIA rendering changes in the future, exporter
        #      should be global for all DraftJSHTMLExporter instances
        self.exporter = HTML(
            {
                "engine": DOM.LXML,
                "style_map": dict(
                    STYLE_MAP,
                    **{
                        INLINE_STYLES.BOLD: "b",
                        INLINE_STYLES.ITALIC: "i",
                        INLINE_STYLES.FALLBACK: self.style_fallback,
                    },
                ),
                "entity_decorators": {
                    ENTITY_TYPES.LINK: self.render_link,
                    ENTITY_TYPES.HORIZONTAL_RULE: lambda props: DOM.create_element("hr"),
                    ENTITY_TYPES.EMBED: self.render_embed,
                    MEDIA: self.render_media,
                    ANNOTATION: self.render_annotation,
                    TABLE: self.render_table,
                },
            }
        )

    @property
    def content_state(self):
        return self.content_editor.content_state

    def render(self):
        blocks = self.content_state["blocks"]
        if not blocks:
            return ""
        if blocks[0].get("text", "-").strip() == "" or blocks[-1].get("text", "-").strip() == "":
            # first and last block may be empty due to client constraints, in this case
            # we must discard them during rendering
            content_state = self.content_state.copy()
            content_state["blocks"] = blocks = blocks[:]
            if blocks[0]["text"].strip() == "" and not blocks[0]["entityRanges"]:
                del blocks[0]
            if blocks and blocks[-1]["text"].strip() == "" and not blocks[-1]["entityRanges"]:
                del blocks[-1]
        else:
            content_state = self.content_state

        try:
            for block in content_state["blocks"]:
                block.setdefault("depth", 0)
                block.setdefault("entityRanges", [])
                block.setdefault("inlineStyleRanges", [])
            html = self.exporter.render(content_state)
        except KeyError as e:
            if e.args == ("text",):
                # "text" may be missing in some case (e.g. comments), and the exporter
                # doesn't support it. To avoid a crash, we render again after
                # filtering out all block elements without "text".
                content_state = self.content_state.copy()
                content_state["blocks"] = [b for b in content_state["blocks"] if "text" in b]
                html = self.exporter.render(content_state)
            else:
                raise e
        # see render_media for details
        return DUMMY_RE.sub("", html)

    def render_annotation(self, props):
        return props["children"]

    def render_media(self, props):
        media_props = props["media"]
        media_type = media_props.get("type", "picture")
        rendition = media_props["renditions"].get("original") or media_props["renditions"]["viewImage"]
        alt_text = media_props.get("alt_text") or ""
        desc = media_props.get("description_text")
        if media_type == "picture":
            embed_type = "Image"
            elt = DOM.create_element("img", {"src": rendition["href"], "alt": alt_text}, props["children"])
        elif media_type == "video":
            embed_type = "Video"
            elt = DOM.create_element(
                "video",
                {"control": "control", "src": rendition["href"], "alt": alt_text, "width": "100%", "height": "100%"},
                props["children"],
            )
        elif media_type == "audio":
            embed_type = "Audio"
            elt = DOM.create_element(
                "audio",
                {"control": "control", "src": rendition["href"], "alt": alt_text, "width": "100%", "height": "100%"},
                props["children"],
            )
        else:
            logger.error("Invalid or not implemented media type: {media_type}".format(media_type=media_type))
            return None

        content = DOM.render(elt)

        if desc:
            content += "<figcaption>{}</figcaption>".format(desc)

        # we need to retrieve the key, there is not straightforward way to do it
        # so we find the key in entityMap with a corresponding value
        embed_key = next(
            k for k, v in self.content_state["entityMap"].items() if v["data"].get("media") == props["media"]
        )

        # <dummy_tag> is needed for the comments, because a root node is necessary
        # it will be removed during rendering.
        embed = DOM.parse_html(
            dedent(
                """\
            <dummy_tag><!-- EMBED START {embed_type} {{id: "editor_{key}"}} -->
            <figure>{content}</figure>
            <!-- EMBED END {embed_type} {{id: "editor_{key}"}} --></dummy_tag>"""
            ).format(embed_type=embed_type, key=embed_key, content=content)
        )

        return embed

    def render_link(self, props):
        if "url" in props:
            attribs = {"href": props["url"]}
        else:
            link_data = props.get("link", {})
            if link_data.get("attachment"):
                attribs = {"data-attachment": link_data["attachment"]}
            elif link_data.get("target"):
                attribs = {"href": link_data["href"], "target": link_data["target"]}
            else:
                attribs = {"href": link_data["href"]}

        return DOM.create_element("a", attribs, props["children"])

    def render_embed(self, props):
        embed_pre_process = app.config.get("EMBED_PRE_PROCESS")
        if embed_pre_process:
            for callback in embed_pre_process:
                callback(props["data"])
        # we use superdesk.etree.parse_html instead of DOM.parse_html as the later modify the content
        # and we use directly the wrapping <div> returned with "content='html'". This works because
        # we use the lxml engine with DraftJSExporter.
        div = parse_html(props["data"]["html"], content="html")
        div.set("class", "embed-block")
        description = props.get("description")
        if description:
            p = DOM.create_element("p", {"class": "embed-block__description"}, description)
            DOM.append_child(div, p)

        return div

    def render_table(self, props):
        num_cols = props["data"]["numCols"]
        num_rows = props["data"]["numRows"]
        with_header = props["data"].get("withHeader", False)
        cells = props["data"]["cells"]
        table = DOM.create_element("table")
        if with_header:
            start_row = 1
            thead = DOM.create_element("thead")
            DOM.append_child(table, thead)
            tr = DOM.create_element("tr")
            DOM.append_child(thead, tr)
            for col_idx in range(num_cols):
                th = DOM.create_element("th")
                DOM.append_child(tr, th)
                try:
                    content_state = cells[0][col_idx]
                except IndexError:
                    continue
                try:
                    content = DOM.parse_html(self.exporter.render(content_state))
                except etree.ParserError:
                    continue
                if content.text or len(content):
                    DOM.append_child(th, content)
        else:
            start_row = 0

        if not with_header or num_rows > 1:
            tbody = DOM.create_element("tbody")
            DOM.append_child(table, tbody)

        for row_idx in range(start_row, num_rows):
            tr = DOM.create_element("tr")
            DOM.append_child(tbody, tr)
            for col_idx in range(num_cols):
                td = DOM.create_element("td")
                DOM.append_child(tr, td)
                try:
                    content_state = cells[row_idx][col_idx]
                except IndexError:
                    continue
                try:
                    content = DOM.parse_html(self.exporter.render(content_state))
                except etree.ParserError:
                    continue
                if content.text or len(content):
                    DOM.append_child(td, content)

        return table

    def style_fallback(self, props):
        type_ = props["inline_style_range"]["style"]
        # we need to use fallback for annotations, has they have suffixes, it's
        # not only "ANNOTATION"
        if type_.startswith("ANNOTATION"):
            attribs = {"annotation-id": type_[11:]}
            return DOM.create_element("span", attribs, props["children"])
        if type_.startswith("COMMENT"):
            # nothing to render for comments
            pass
        else:
            logger.error("No style renderer for {type_!r}".format(type_=type_))


class Editor3Content(EditorContent):
    """Handle content for Superdesk Editor 3, using Draft.js ContentState

    see https://medium.com/@rajaraodv/how-draft-js-represents-rich-text-data-eeabb5f25cf2 for documentation
    on ContentState.
    """

    editor_version = 3
    HTML_EXPORTER = DraftJSHTMLExporter

    def __init__(self, item, field="body_html", is_html=True, reload=False):
        """
        :param item: item containing Draft.js ContentState
        :param field: field to manage, can be "body_html", "headline", etc.
        :param is_html: boolean to indicate if the field is html or text field
        :param reload: boolean to indicate if the content state should be reloaded from item value
        """
        self.item = item
        self.field = field
        self.is_html = is_html
        self.content_state = get_field_content_state(item, field)
        if not self.content_state or reload:
            _html = get_field_value(item, field)
            self._create_state_from_html(_html)
        self.blocks = BlockSequence(self)
        self.html_exporter = DraftJSHTMLExporter(self)

    def _create_state_from_html(self, value=None):
        self.content_state = {
            "blocks": [],
            "entityMap": {},
        }

        if not value:
            return

        def create_entity(entity_type, data, mutability="MUTABLE"):
            key = len(self.content_state["entityMap"].keys())
            self.content_state["entityMap"][str(key)] = {
                "type": entity_type,
                "mutability": mutability,
                "data": data,
            }
            return key

        def create_atomic_block(entity_type, data):
            block = self.create_block("atomic", text=" ").data
            entity_key = create_entity(entity_type, data)
            block["entityRanges"] = [{"offset": 0, "length": 1, "key": entity_key}]
            block["inlineStyleRanges"] = []
            self.content_state["blocks"].append(block)

        if self.is_html:
            root = parse_html(value, "html")
            for i, elem in enumerate(root):
                try:
                    block_type = next((key for key, val in BLOCK_MAP.items() if val == elem.tag))
                    depth = 0
                except StopIteration:
                    block_type = None
                    depth = 0
                    if elem.tag == "figure":
                        try:
                            m = re.search(r'<!-- EMBED START (?:Image|Video) {id: "(.*)"}', str(root[i - 1]).strip())
                            media = self.item["associations"][m.group(1)]
                            create_atomic_block("MEDIA", {"media": media})
                        except (KeyError, IndexError, AttributeError):
                            create_atomic_block("EMBED", {"data": {"html": to_string(elem, method="html")}})
                        continue
                    elif elem.tag in ("ul", "ol"):
                        pass  # generate block for each li
                    elif elem.text and "<!-- EMBED" in str(elem):
                        continue
                    elif elem.tag == "table":
                        data = {"numCols": 0, "numRows": 0, "withHeader": False, "cells": {}}
                        for row, tr in enumerate(elem.iter("tr")):
                            data["numRows"] += 1
                            data["cells"][row] = {}
                            if row == 0 and len(tr):
                                data["numCols"] = len(tr)
                                data["withHeader"] = tr[0].tag == "th"
                            for col, td in enumerate(tr):
                                data["cells"][row][col] = {
                                    "blocks": [
                                        self.create_block("unstyled", text="".join(td.itertext())).data,
                                    ],
                                    "entityMap": {},
                                }
                        create_atomic_block("TABLE", {"data": data})
                        continue
                    elif elem.tag == "div":
                        html = "".join([to_string(child, method="html") for child in elem])
                        if html.startswith("<html><head>"):
                            html = re.sub(
                                r"<\/head><\/html>",
                                "",
                                html.replace("<html><head>", "", 1),
                            )
                        create_atomic_block("EMBED", {"data": {"html": html}})
                        continue
                    else:
                        logger.warning("ignore block %s", str(elem.tag))
                        continue
                block_text = elem.text or ""
                inline_style_ranges = []
                entity_ranges = []
                for child in elem:
                    child_text = "".join(child.itertext())

                    if child.tag in TAG_STYLE_MAP:
                        inline_style_ranges.append(
                            {
                                "offset": len(block_text),
                                "length": len(child_text),
                                "style": TAG_STYLE_MAP[child.tag],
                            }
                        )

                    if child.tag in TAG_ENTITY_MAP:
                        if not child_text:
                            child_text = " "  # must be non-empty
                        if child.tag == "a":
                            data = {"link": {"href": child.attrib.get("href"), "target": child.attrib.get("target")}}
                        else:
                            data = {}
                        entity_key = create_entity(TAG_ENTITY_MAP[child.tag], data)
                        entity_ranges.append({"offset": len(block_text), "length": len(child_text), "key": entity_key})

                    if child.tag == "li":
                        child_type = (
                            BLOCK_TYPES.UNORDERED_LIST_ITEM if elem.tag == "ul" else BLOCK_TYPES.ORDERED_LIST_ITEM
                        )
                        block = self.create_block(child_type, text=child_text).data
                        block.update(depth=depth)
                        self.content_state["blocks"].append(block)

                    block_text += child_text

                    if child.tail and child.tail.strip():
                        block_text += child.tail

                if elem.tail and elem.tail.strip():
                    block_text += elem.tail

                if block_type:  # no block type for ul/ol
                    block = self.create_block(block_type, text=block_text).data
                    block["inlineStyleRanges"] = inline_style_ranges
                    block["entityRanges"] = entity_ranges
                    self.content_state["blocks"].append(block)
        else:
            for line in value.split("\n"):
                self.content_state["blocks"].append(self.create_block(BLOCK_TYPES.UNSTYLED, text=line).data)

    @property
    def html(self):
        exported = self.html_exporter.render()
        pieces = lxml_html.fragments_fromstring(exported)
        return "\n".join([render_fragment(elem).strip() for elem in pieces if elem is not None]).strip()

    @property
    def text(self):
        return "\n".join([block.text for block in self.blocks]).strip()

    def get_next_entity_key(self):
        """Return a non existing key for entityMap"""
        return max((int(k) for k in self.content_state["entityMap"].keys()), default=-1) + 1

    def update_item(self):
        value = self.html if self.is_html else self.text
        set_field_value(self.item, self.field, value)
        set_field_content_state(self.item, self.field, self.content_state)

    def create_block(self, block_type, *args, **kwargs):
        cls_name = "{}Block".format(block_type.capitalize())
        if cls_name in globals():
            cls = globals()[cls_name]
            return cls(self, *args, **kwargs)
        else:
            block = Block(self, *args, **kwargs)
            block.data["type"] = block_type
            return block

    def set_blocks(self, blocks):
        try:
            data = self.blocks[0].data.get("data")  # store internal data from first block
        except IndexError:
            data = {}
        self.content_state["blocks"] = [getattr(block, "data", block) for block in blocks]
        self.blocks = BlockSequence(self)
        if not len(self.blocks):
            self.prepend("Unstyled")
        if not self.blocks[0].data.get("data") and data:
            self.blocks[0].data["data"] = data

    def prepend(self, block_type, *args, **kwargs):
        """Shortcut to prepend a block from its type"""
        block = self.create_block(block_type, *args, **kwargs)
        self.prepend_block(block)
        return block

    def prepend_block(self, block):
        # first block may be empty, this is a client constraint. In this case,
        # we prepend our block after it
        if self.blocks and not self.blocks[0].text.strip():
            index = 1
        else:
            index = 0
        self.blocks.insert(index, block)


def _replace_text(content_state, old, new):
    if not old:
        return
    for block in content_state["blocks"]:
        if block.get("type") == "atomic":
            entity = content_state[ENTITY_MAP][str(block[ENTITY_RANGES][0]["key"])]
            if entity["type"] == "TABLE":
                cells = entity["data"]["data"]["cells"]
                for row in cells.values():
                    for cell in row.values():
                        _replace_text(cell, old, new)
            continue
        if not block.get("text"):
            continue
        end = 0
        while True:
            try:
                index = block["text"].index(old, end)
                end = index + len(old)
                block["text"] = new.join([block["text"][:index], block["text"][end:]])
                for range_field in (ENTITY_RANGES, INLINE_STYLE_RANGES):
                    if block.get(range_field):
                        ranges = []
                        for range_ in block[range_field]:
                            range_end = range_["offset"] + range_["length"]
                            if range_["offset"] > end:  # starting after replaced, move it
                                range_["offset"] += len(new) - len(old)
                                ranges.append(range_)
                            elif range_end <= index:  # starting before replaced text, keep it
                                ranges.append(range_)
                            elif range_["offset"] <= index and range_end >= end:  # contain the text, fix length
                                range_["length"] += len(new) - len(old)
                                ranges.append(range_)
                            else:
                                # remove ranges overlapping with replaced text
                                if range_field == ENTITY_RANGES:
                                    content_state["entityMap"].pop(str(range_["key"]))
                                continue
                        block[range_field] = ranges
            except ValueError:
                break


def replace_text(item, field, old, new, is_html=True):
    """Replace all occurences of old replaced with new.

    It won't replace it in atomic blocks and embeds,
    only text blocks, headings, tables, ul/ol.
    """
    editor = Editor3Content(item, field, is_html)
    _replace_text(editor.content_state, old, new)
    editor.update_item()


def filter_blocks(item, field, filter, is_html=True):
    """Filter content blocks for field.

    It will keep only blocks for which filter returns True.
    """
    editor = Editor3Content(item, field, is_html)
    blocks = []
    for block in editor.blocks:
        if filter(block):
            blocks.append(block)
    editor.set_blocks(blocks)
    editor.update_item()


def generate_fields(item, fields=None, force=False):
    """Generate item fields from editor states

    :param item: item containing Draft.js ContentState
    :param fields: fields to generate, None to generate all fields with a content state
    :param force: force refreshing of content state from item field
    """
    if fields is None:
        fields = get_content_state_fields(item)

    for field in fields:
        old_field = None
        if CHECK_GENERATE_CONSISTENCY:
            old_field = item.get(field)
        editor = Editor3Content(item, field, is_html=is_html(field), reload=force)
        editor.update_item()
        if CHECK_GENERATE_CONSISTENCY and not force:
            if old_field is not None and old_field.strip() != item[field].strip():
                logger.warning(
                    "Generated HTML inconsistency between client and backend, we'll use client one",
                    extra=dict(
                        client=old_field,
                        backend=item[field],
                        field_state=get_field_content_state(item, field),
                    ),
                )
                item[field] = old_field


def is_html(field) -> bool:
    if "extra" in field:
        field_id = field.split(">")[1]
        field_options = superdesk.get_resource_service("vocabularies").get_field_options(field_id)
        return field_options.get("single") is not True
    return field not in TEXT_FIELDS


def render_fragment(elem) -> str:
    if elem.tag == "p" and not elem.text and not len(elem):
        # client renders empty paragraph as `<p><br></p>`
        etree.SubElement(elem, "br", nsmap=None, attrib=None)
    return str(lxml_html.tostring(elem, encoding="unicode"))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""The Destinations API allows to retrieve the list of ``StorageDestinations``.


=====================   =======================================================
**endpoint name**        'storage_destinations'
**resource title**       'Destinations'
**resource url**         [GET] '/sams/destinations'
**item url**             [GET] '/sams/destinations/<:class:`str`>'
**schema**               :attr:`sams_client.schemas.destinationSchema`
=====================   =======================================================
"""

import logging
import superdesk
from .client import get_sams_client

logger = logging.getLogger(__name__)
destinations_bp = superdesk.Blueprint("sams_destinations", __name__)


@destinations_bp.route("/sams/destinations", methods=["GET"])
def get():
    """
    Returns a list of all the registered storage destinations
    """
    destinations = get_sams_client().destinations.search()
    return destinations.json(), destinations.status_code


@destinations_bp.route("/sams/destinations/<item_id>", methods=["GET"])
def find_one(item_id):
    """
    Uses item_id and returns the destination
    name and provider name of the respective storage destination
    """
    item = get_sams_client().destinations.get_by_id(item_id=item_id)
    return item.json(), item.status_code



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging

import superdesk
from .vidible import bp

logger = logging.getLogger(__name__)


def init_app(app) -> None:
    superdesk.blueprint(bp, app)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""Privileges registry."""
from typing import Optional
from .errors import PrivilegeNameError
from flask_babel.speaklater import LazyString

import superdesk

_privileges = {}
_intrinsic_privileges = {}


GLOBAL_SEARCH_PRIVILEGE = "use_global_saved_searches"


def privilege(
    name,
    label: Optional[LazyString] = None,
    description: Optional[LazyString] = None,
    category: Optional[LazyString] = None,
    **kwargs,
):
    """Register privilege.

    Privilege name must not contain "."

    Privilege properties:
    - name
    - label
    - description
    - category
    """
    if "." in name:
        raise PrivilegeNameError('"." is not supported in privilege name "%s"' % name)
    _privileges[name] = kwargs
    _privileges[name].update(
        dict(
            name=name,
            label=label,
            category=category,
            description=description,
        )
    )


def get_item_privilege_name(resource: str, item):
    return "resource:{}:{}".format(resource, item["_id"])


def _get_resource_privileges():
    resource_privileges = []
    for name, resource in superdesk.resources.items():
        if not getattr(resource, "item_privileges", None):
            continue
        items = superdesk.get_resource_service(name).get(None, {})
        for item in items:
            try:
                label = resource.item_privileges_label.format(**item)
            except KeyError:
                label = "{}: {}".format(name, item["_id"])
            resource_privileges.append(
                {
                    "name": get_item_privilege_name(name, item),
                    "label": label,
                    "category": name,
                }
            )
    return resource_privileges


def _get_registered_privileges():
    """Get list of all registered privileges."""
    return [v for v in _privileges.values()]


def get_privilege_list():
    return _get_registered_privileges() + _get_resource_privileges()


def intrinsic_privilege(resource_name, method=None):
    """
    Registers intrinsic privileges.
    """

    if method is None:
        method = []

    _intrinsic_privileges[resource_name] = method


def get_intrinsic_privileges():
    """Get list of all registered intrinsic privileges."""

    return _intrinsic_privileges



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2020 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from typing import Dict, Any, Optional

from bson import ObjectId
from eve.utils import str_to_date
from requests import Response
from requests.exceptions import HTTPError

from superdesk.errors import SuperdeskApiError
from superdesk.storage.superdesk_file import SuperdeskFile
from sams_client import SamsClient


class SAMSFileWrapper(SuperdeskFile):
    """Helper class to convert SAMS Asset Binary response to file-like object"""

    def __init__(self, asset: Dict[str, Any], file: Response):
        super().__init__()
        self.write(file.content)
        self.seek(0)
        self.content_type = asset["mimetype"]
        self.length = asset["length"]
        self._name = asset["name"]
        self.filename = asset["filename"]
        self.upload_date = str_to_date(asset["_updated"])
        self.md5 = asset["_etag"]
        self._id = asset["_id"]


def get_asset_from_sams(client: SamsClient, asset_id: ObjectId) -> Optional[Dict[str, Any]]:
    """Attempts to retrieve an Asset from SAMS"""

    response = client.assets.get_by_id(asset_id)

    if response.status_code == 404:
        return None

    raise_sams_error(response)
    return response.json()


def get_file_from_sams(client: SamsClient, asset_id: ObjectId) -> Optional[SAMSFileWrapper]:
    """Attempts to retrieve the Asset Binary from SAMS"""

    asset = get_asset_from_sams(client, asset_id)

    if asset:
        response = client.assets.get_binary_by_id(asset_id)

        # If the Asset exists in SAMS, then so should the file
        raise_sams_error(response)

        return SAMSFileWrapper(asset, response)

    return None


def get_asset_public_url(client: SamsClient, asset_id: ObjectId) -> Optional[str]:
    """Attempts to retrieve the public URL for the Asset"""

    asset = get_asset_from_sams(client, asset_id)

    if asset:
        return (asset["_links"].get("public") or {}).get("href")

    return None


def get_default_set_id_for_upload(client: SamsClient, data: Dict[str, Any]) -> ObjectId:
    """Returns the Set ID to use for a file upload"""

    if not data.get("set_id"):
        # TODO: Implement default sets for Desk/Module (i.e. Planning)
        #       For now, simply return the first Set found
        response = client.sets.search(params={"max_results": 1})
        raise_sams_error(response)
        set_item = response.json()["_items"][0]
        return ObjectId(set_item["_id"])

    return ObjectId(data["set_id"])


def raise_sams_error(response: Response):
    """If an error was raised from SAMS, attempts to re-raise it as a Superdesk error

    Raising Superdesk errors are to be used outside the SAMS workspace functionality
    i.e. with the Eve MediaStorage provider functionality
    """

    try:
        response.raise_for_status()
    except HTTPError as http_error:
        error = response.json()
        raise SuperdeskApiError(
            status_code=response.status_code,
            message=error.get("description") or "",
            payload=error,
            exception=http_error,
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from ..service import ProdApiService


class ItemsService(ProdApiService):
    excluded_fields = {
        "fields_meta",
        "unique_id",
        "family_id",
        "event_id",
        "lock_session",
        "lock_action",
        "lock_time",
        "lock_user",
    } | ProdApiService.excluded_fields



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from typing import Any
import superdesk
from superdesk.resource import Resource
from superdesk.services import BaseService
from apps.comments import CommentsService, CommentsResource, comments_schema
from superdesk.errors import SuperdeskApiError


comments_schema = dict(comments_schema)
comments_schema.update({"item": Resource.rel("archive", True, True, type="string")})


class ItemCommentsResource(CommentsResource):
    schema = comments_schema
    resource_methods = ["GET", "POST", "DELETE"]
    datasource = {"default_sort": [("_created", -1)]}
    privileges = {"POST": "archive", "DELETE": "archive"}


class ItemCommentsService(CommentsService):
    notification_key = "item:comment"


class ItemCommentsSubResource(Resource):
    url = "archive/<path:item>/comments"
    schema = comments_schema
    datasource = {"source": "item_comments"}
    resource_methods = ["GET"]


class ItemCommentsSubService(BaseService):
    def check_item_valid(self, item_id):
        item = superdesk.get_resource_service("archive").find_one(req=None, _id=item_id)
        if not item:
            msg = "Invalid content item ID provided: %s" % item_id
            raise SuperdeskApiError.notFoundError(msg)

    def get(self, req, lookup):
        self.check_item_valid(lookup.get("item"))
        return super().get(req, lookup)


def init_app(app) -> None:
    endpoint_name = "item_comments"
    service: Any = ItemCommentsService(endpoint_name, backend=superdesk.get_backend())
    ItemCommentsResource(endpoint_name, app=app, service=service)

    endpoint_name = "content_item_comments"
    service = ItemCommentsSubService(endpoint_name, backend=superdesk.get_backend())
    ItemCommentsSubResource(endpoint_name, app=app, service=service)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2020 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from typing import Dict, Any, BinaryIO, Union
import logging
from os import path

from eve.flaskapp import Eve
from eve.io.mongo.media import MediaStorage
from bson import ObjectId
from flask import current_app, request
from werkzeug.utils import secure_filename

from superdesk.factory.app import get_media_storage_class
from superdesk.default_settings import strtobool
from superdesk.storage.mimetype_mixin import MimetypeMixin
from superdesk.storage.superdesk_file import SuperdeskFile
from superdesk.attachments import RESOURCE as ATTACHMENTS_RESOURCE
from superdesk.sams import get_sams_client

from sams_client import SamsClient

from .utils import (
    get_asset_from_sams,
    get_file_from_sams,
    raise_sams_error,
    get_default_set_id_for_upload,
    get_asset_public_url,
)

logger = logging.getLogger(__name__)


SAMS_RESOURCE_ENABLED: Dict[str, bool] = {ATTACHMENTS_RESOURCE: True}


def get_sams_values_from_resource_schema(resource: str, data: Dict[str, Any]):
    schema = current_app.config["DOMAIN"][resource]["schema"]

    def _get_field(field: str):
        sams_mapping = (schema.get(field) or {}).get("sams") or {}
        return sams_mapping.get("field") or field

    def _get_value(field: str):
        value_type = (schema.get(field) or {}).get("type")
        sams_mapping = (schema.get(field) or {}).get("sams") or {}
        data_value = data[field]

        if value_type == "boolean" and isinstance(data_value, str):
            data_value = strtobool(data_value)

        return (sams_mapping.get("map_value") or {}).get(data_value) or data_value

    return {_get_field(field): _get_value(field) for field in data.keys()}


class SAMSMediaStorage(MediaStorage, MimetypeMixin):
    """SAMS MediaStorage implementation for Eve

    Uses SAMS if the requested resource exists in `SAMS_RESOURCE_ENABLED`
    Otherwise the fallback MediaStorage is used (`SuperdeskGridFSMediaStorage` or `AmazonMediaStorage`)
    """

    def __init__(self, app: Eve = None):
        super(SAMSMediaStorage, self).__init__(app or current_app)

        fallback_klass = get_media_storage_class(self.app.config, False)
        self._fallback = fallback_klass(self.app)
        self._client: SamsClient = get_sams_client(self.app)

    def url_for_external(self, media_id: str, resource: str = None) -> str:
        """Returns a URL for external use

        Returns a URL for use with the SAMS FileServer (if the Asset is public),
        otherwise falls back to the Content/Production API (via MediaStorage fallback)

        :param str media_id: The ID of the asset
        :param str resource: The name of the resource type this Asset is attached to
        :rtype: str
        :return: The URL for external use
        """

        if resource is not None and SAMS_RESOURCE_ENABLED.get(resource, False):
            url = get_asset_public_url(self._client, ObjectId(media_id))

            if url:
                # If SAMS provided a public URL, return that
                # otherwise return a href for Production/Content API instead
                return url

        return self._fallback.url_for_external(media_id)

    def get(self, id_or_filename: Union[ObjectId, str], resource: str = None, **kwargs) -> SuperdeskFile:
        """Attempts to retrieve the file from SAMS or the `_fallback` provider"""

        if resource is not None and SAMS_RESOURCE_ENABLED.get(resource, False):
            file = get_file_from_sams(self._client, ObjectId(id_or_filename))
            if file:
                return file

        return self._fallback.get(id_or_filename, resource=resource, **kwargs)

    def put(
        self,
        content: BinaryIO,
        filename: str = None,
        content_type: str = None,
        resource: str = None,
        **kwargs: Dict[str, Any],
    ) -> ObjectId:
        """Attempts to upload the file to SAMS or the `_fallback` provider"""

        if resource is not None and SAMS_RESOURCE_ENABLED.get(resource, False):
            data = request.form.to_dict()
            metadata = get_sams_values_from_resource_schema(resource, data)

            if not metadata.get("set_id"):
                metadata["set_id"] = get_default_set_id_for_upload(self._client, data)
            if not metadata.get("state"):
                metadata["state"] = "public"
            if not metadata.get("filename"):
                metadata["filename"] = secure_filename(path.basename(filename)) if filename else ""

            response = self._client.assets.create(docs=metadata, files={"binary": content})
            raise_sams_error(response)
            return response.json()["_id"]

        return self._fallback.put(content, filename=filename, content_type=content_type, resource=resource, **kwargs)

    def delete(self, id_or_filename: Union[ObjectId, str], resource: str = None, **kwargs):
        """Ignored if the file is stored in SAMS, otherwise deletes from the `_fallback` provider"""

        if resource is not None and SAMS_RESOURCE_ENABLED.get(resource, False):
            if get_asset_from_sams(self._client, ObjectId(id_or_filename)):
                # If the Asset exists in SAMS, then we should not delete it
                # as Assets should be deleted using DELETE /sams/assets/<id>
                return

        self._fallback.delete(id_or_filename, resource=resource, **kwargs)

    def exists(self, id_or_filename: Union[ObjectId, str], resource: str = None, **kwargs) -> bool:
        """Returns True if the file is available in SAMS or the `_fallback` provider"""

        if resource is not None and SAMS_RESOURCE_ENABLED.get(resource, False):
            if get_asset_from_sams(self._client, ObjectId(id_or_filename)):
                return True

        return self._fallback.exists(id_or_filename, resource=resource, **kwargs)

    def __getattr__(self, name: Any) -> Any:
        """Returns `_fallback` attribute if not available in SAMSMediaStorage"""

        return self._fallback.__getattribute__(name)



import superdesk
from superdesk.tests import TestCase


class VocabulariesServiceGetItemsTestCase(TestCase):
    def setUp(self):
        self.app.data.insert(
            "vocabularies",
            [
                {
                    "_id": "funny-keywords",
                    "items": [
                        {
                            "name": "BRIEF",
                            "qcode": "BRIEF",
                            "is_active": False,
                            "translations": {"name": {"nl": "BRIEF", "fr": "BRIEF"}},
                        },
                        {
                            "name": "PREVIEW",
                            "qcode": "PREVIEW",
                            "is_active": True,
                            "translations": {"name": {"nl": "Voorbericht", "fr": "Avant-Papier"}},
                        },
                        {
                            "name": "WEST FLANDERS",
                            "qcode": "WEST FLANDERS",
                            "is_active": True,
                            "translations": {"name": {"nl": "West-Vlaanderen", "fr": "Flandre occidentale"}},
                        },
                        {
                            "name": "EAST FLANDERS",
                            "qcode": "EAST FLANDERS",
                            "is_active": False,
                            "translations": {"name": {"nl": "Oost-Vlaanderen", "fr": "Flandre orientale"}},
                        },
                        {
                            "name": "ANTWERP",
                            "qcode": "ANTWERP",
                            "is_active": True,
                            "translations": {"name": {"nl": "Antwerpen", "fr": "Anvers"}},
                        },
                    ],
                },
                {
                    "_id": "sad-keywords",
                    "items": [
                        {
                            "name": "WALLOON BRABANT",
                            "qcode": "WALLOON BRABANT",
                            "is_active": True,
                            "translations": {"name": {"nl": "Waals-Brabant", "fr": "Brabant wallon"}},
                        },
                        {
                            "name": "BIOGRAPHY",
                            "qcode": "BIOGRAPHY",
                            "is_active": True,
                            "translations": {"name": {"nl": "Biografie", "fr": "Biographie"}},
                        },
                        {
                            "name": "WEATHER",
                            "qcode": "WEATHER",
                            "is_active": True,
                            "translations": {"name": {"nl": "Weer", "fr": "Météo"}},
                        },
                    ],
                },
            ],
        )

    def test_search_by_id(self):
        items = superdesk.get_resource_service("vocabularies").get_items(_id="funny-keywords")
        self.assertEqual(len(items), 3)

        items = superdesk.get_resource_service("vocabularies").get_items(_id="sad-keywords")
        self.assertEqual(len(items), 3)

        items = superdesk.get_resource_service("vocabularies").get_items(_id="not-found-id")
        self.assertEqual(items, [])

    def test_search_by_qcode(self):
        items = superdesk.get_resource_service("vocabularies").get_items(_id="funny-keywords", qcode="PREVIEW")
        self.assertEqual(
            items,
            [
                {
                    "name": "PREVIEW",
                    "qcode": "PREVIEW",
                    "scheme": "funny-keywords",
                    "translations": {"name": {"nl": "Voorbericht", "fr": "Avant-Papier"}},
                }
            ],
        )
        items = superdesk.get_resource_service("vocabularies").get_items(_id="funny-keywords", qcode="not found")
        self.assertEqual(items, [])

    def test_search_by_is_active(self):
        items = superdesk.get_resource_service("vocabularies").get_items(_id="funny-keywords", is_active=False)
        self.assertEqual(len(items), 2)

        items = superdesk.get_resource_service("vocabularies").get_items(_id="funny-keywords", is_active=True)
        self.assertEqual(len(items), 3)

    def test_search_by_name(self):
        items = superdesk.get_resource_service("vocabularies").get_items(
            _id="funny-keywords", name="BRIEF", is_active=False
        )
        self.assertEqual(
            items,
            [
                {
                    "name": "BRIEF",
                    "qcode": "BRIEF",
                    "scheme": "funny-keywords",
                    "translations": {"name": {"nl": "BRIEF", "fr": "BRIEF"}},
                }
            ],
        )

        items = superdesk.get_resource_service("vocabularies").get_items(_id="funny-keywords", name="not found")
        self.assertEqual(items, [])

    def test_search_by_translation_name(self):
        items = superdesk.get_resource_service("vocabularies").get_items(
            _id="funny-keywords",
            name="Avant-Papier",
            lang="fr",
        )
        self.assertEqual(
            items,
            [
                {
                    "name": "PREVIEW",
                    "qcode": "PREVIEW",
                    "scheme": "funny-keywords",
                    "translations": {"name": {"nl": "Voorbericht", "fr": "Avant-Papier"}},
                }
            ],
        )

        items = superdesk.get_resource_service("vocabularies").get_items(
            _id="funny-keywords",
            name="AVANT-PAPIER",
            lang="fr",
        )
        self.assertEqual(
            items,
            [
                {
                    "name": "PREVIEW",
                    "qcode": "PREVIEW",
                    "scheme": "funny-keywords",
                    "translations": {"name": {"nl": "Voorbericht", "fr": "Avant-Papier"}},
                }
            ],
        )

        items = superdesk.get_resource_service("vocabularies").get_items(
            _id="funny-keywords",
            name="Avant-Papier-bla-bla",
            lang="fr",
        )
        self.assertEqual(items, [])

    def test_search_by_all_fields(self):
        items = superdesk.get_resource_service("vocabularies").get_items(
            _id="funny-keywords",
            qcode="PREVIEW",
            name="Avant-Papier",
            lang="fr",
            is_active=True,
        )
        self.assertEqual(
            items,
            [
                {
                    "name": "PREVIEW",
                    "qcode": "PREVIEW",
                    "scheme": "funny-keywords",
                    "translations": {"name": {"nl": "Voorbericht", "fr": "Avant-Papier"}},
                }
            ],
        )

        items = superdesk.get_resource_service("vocabularies").get_items(
            _id="funny-keywords",
            qcode="PREVIEW",
            name="Avant-Papier",
            lang="fr",
            is_active=False,
        )
        self.assertEqual(items, [])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource


class SystemMessagesResource(Resource):
    schema = {
        "is_active": {"type": "boolean", "default": False},
        "type": {
            "type": "string",
            "allowed": ["warning", "alert", "primary", "success", "highlight", "light"],
            "required": True,
        },
        "message_title": {"type": "string", "required": True},
        "message": {"type": "string", "required": True, "maxlength": 200},
        "user_id": Resource.rel("users"),
    }

    resource_methods = ["GET", "POST"]
    item_methods = ["GET", "PATCH", "DELETE"]
    privileges = {"POST": "system_messages", "PATCH": "system_messages", "DELETE": "system_messages"}



from flask_babel._compat import string_types as string_types
from flask_babel.speaklater import LazyString as LazyString
from typing import Any, Optional

class Babel:
    default_date_formats: Any = ...
    app: Any = ...
    locale_selector_func: Any = ...
    timezone_selector_func: Any = ...
    def __init__(self, app: Optional[Any] = ..., default_locale: str = ..., default_timezone: str = ..., default_domain: str = ..., date_formats: Optional[Any] = ..., configure_jinja: bool = ...) -> None: ...
    date_formats: Any = ...
    def init_app(self, app: Any): ...
    def localeselector(self, f: Any): ...
    def timezoneselector(self, f: Any): ...
    def list_translations(self): ...
    @property
    def default_locale(self): ...
    @property
    def default_timezone(self): ...
    @property
    def domain(self): ...
    @property
    def translation_directories(self) -> None: ...

def _(string: Any, **variables: Any): ...
def get_translations(): ...
def get_locale(): ...
def get_timezone(): ...
def refresh() -> None: ...
def force_locale(locale: Any) -> None: ...
def to_user_timezone(datetime: Any): ...
def to_utc(datetime: Any): ...
def format_datetime(datetime: Optional[Any] = ..., format: Optional[Any] = ..., rebase: bool = ...): ...
def format_date(date: Optional[Any] = ..., format: Optional[Any] = ..., rebase: bool = ...): ...
def format_time(time: Optional[Any] = ..., format: Optional[Any] = ..., rebase: bool = ...): ...
def format_timedelta(datetime_or_timedelta: Any, granularity: str = ..., add_direction: bool = ..., threshold: float = ...): ...
def format_number(number: Any): ...
def format_decimal(number: Any, format: Optional[Any] = ...): ...
def format_currency(number: Any, currency: Any, format: Optional[Any] = ..., currency_digits: bool = ..., format_type: str = ...): ...
def format_percent(number: Any, format: Optional[Any] = ...): ...
def format_scientific(number: Any, format: Optional[Any] = ...): ...
def gettext(string: Any, **variables: Any): ...
def ngettext(singular: Any, plural: Any, num: Any, **variables: Any): ...
def pgettext(context: Any, string: Any, **variables: Any): ...
def npgettext(context: Any, singular: Any, plural: Any, num: Any, **variables: Any): ...
def lazy_gettext(string: Any, **variables: Any) -> LazyString: ...
def lazy_ngettext(singular: Any, plural: Any, num: Any, **variables: Any): ...
def lazy_pgettext(context: Any, string: Any, **variables: Any): ...



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""The Assets API to retrieve, create, update, delete the list of ``Assets``.


=====================   =================================================
**endpoint name**        'assets'
**resource title**       'Assets'
**resource url**         [GET] '/sams/assets'
**item url**             [GET] '/sams/assets/<:class:`str`>'
**schema**               :attr:`sams_client.schemas.ASSET_SCHEMA`
=====================   =================================================
"""

import ast
import logging
import superdesk
from flask import request
from superdesk.errors import SuperdeskApiError
from superdesk.notification import push_notification
from .utils import get_file_from_sams
from superdesk.storage.superdesk_file import generate_response_for_file
from apps.auth import get_auth, get_user_id
from .client import get_sams_client

logger = logging.getLogger(__name__)
assets_bp = superdesk.Blueprint("sams_assets", __name__)


@assets_bp.route("/sams/assets", methods=["GET"])
def get():
    """
    Returns a list of all the registered assets
    """
    assets = get_sams_client().assets.search(params=request.args.to_dict())
    return assets.json(), assets.status_code


@assets_bp.route("/sams/assets/<item_id>", methods=["GET"])
def find_one(item_id):
    """
    Uses item_id and returns the corresponding
    asset
    """
    item = get_sams_client().assets.get_by_id(item_id=item_id)
    return item.json(), item.status_code


@assets_bp.route("/sams/assets/binary/<item_id>", methods=["GET"])
def get_binary(item_id):
    """
    Uses item_id and returns the corresponding
    asset binary
    """
    file = get_file_from_sams(get_sams_client(), item_id)
    return generate_response_for_file(file)


@assets_bp.route("/sams/assets", methods=["POST"])
def create():
    """
    Creates new Asset
    """
    files = {"binary": request.files["binary"]}
    docs = request.form.to_dict()
    post_response = get_sams_client().assets.create(docs=docs, files=files, external_user_id=get_user_id(True))
    if post_response.status_code == 201:
        push_notification(
            "sams:asset:created",
            item_id=post_response.json()["_id"],
            user_id=get_user_id(True),
            session_id=get_auth()["_id"],
            _etag=post_response.json()["_etag"],
            extension="sams",
        )
    return post_response.json(), post_response.status_code


@assets_bp.route("/sams/assets/<item_id>", methods=["DELETE"])
def delete(item_id):
    """
    Uses item_id and deletes the corresponding asset
    """
    try:
        etag = request.headers["If-Match"]
    except KeyError:
        raise SuperdeskApiError.badRequestError("If-Match field missing in header")

    delete_response = get_sams_client().assets.delete(item_id=item_id, headers={"If-Match": etag})
    if delete_response.status_code != 204:
        return delete_response.json(), delete_response.status_code
    if delete_response.status_code == 204:
        push_notification(
            "sams:asset:deleted",
            item_id=item_id,
            user_id=get_user_id(True),
            session_id=get_auth()["_id"],
            extension="sams",
        )
    return "", delete_response.status_code


@assets_bp.route("/sams/assets/<item_id>", methods=["PATCH"])
def update(item_id):
    """
    Uses item_id and updates the corresponding asset
    """
    try:
        etag = request.headers["If-Match"]
    except KeyError:
        raise SuperdeskApiError.badRequestError("If-Match field missing in header")

    if request.files.get("binary"):
        # The binary data was supplied so this must be a multipart request
        # Get the updates from the `request.form` attribute
        files = {"binary": request.files["binary"]}
        updates = request.form.to_dict()
    else:
        # Only the metadata was supplied so this must be a standard JSON request
        # Get the updates from the `request.get_json` function
        files = {}
        updates = request.get_json()

    update_response = get_sams_client().assets.update(
        item_id=item_id, updates=updates, headers={"If-Match": etag}, files=files, external_user_id=get_user_id(True)
    )
    if update_response.status_code == 200:
        push_notification(
            "sams:asset:updated",
            item_id=update_response.json()["_id"],
            user_id=get_user_id(True),
            session_id=get_auth()["_id"],
            _etag=update_response.json()["_etag"],
            extension="sams",
        )
    return update_response.json(), update_response.status_code


@assets_bp.route("/sams/assets/counts", methods=["GET"], defaults={"set_ids": None})
@assets_bp.route("/sams/assets/counts/<set_ids>", methods=["GET"])
def get_assets_count(set_ids):
    set_ids = ast.literal_eval(set_ids) if set_ids else None
    counts = get_sams_client().assets.get_assets_count(set_ids=set_ids)
    return counts


@assets_bp.route("/sams/assets/compressed_binary/<asset_ids>", methods=["GET"])
def get_assets_compressed_binary(asset_ids):
    asset_ids = ast.literal_eval(asset_ids) if asset_ids else None
    zip_binary = get_sams_client().assets.get_binary_zip_by_id(item_ids=asset_ids)
    return zip_binary.content


@assets_bp.route("/sams/assets/lock/<asset_id>", methods=["PATCH"])
def lock_asset(asset_id):
    docs = request.json
    lock_asset_response = get_sams_client().assets.lock_asset(
        item_id=asset_id, external_user_id=get_user_id(True), external_session_id=get_auth()["_id"], docs=docs
    )
    if lock_asset_response.status_code == 200:
        push_notification(
            "sams:asset:lock_asset",
            item_id=asset_id,
            user_id=get_user_id(True),
            session_id=get_auth()["_id"],
            _etag=lock_asset_response.json()["_etag"],
            extension="sams",
        )
    return lock_asset_response.json(), lock_asset_response.status_code


@assets_bp.route("/sams/assets/unlock/<asset_id>", methods=["PATCH"])
def unlock_asset(asset_id):
    docs = request.json
    unlock_asset_response = get_sams_client().assets.unlock_asset(
        item_id=asset_id, external_user_id=get_user_id(True), external_session_id=get_auth()["_id"], docs=docs
    )
    if unlock_asset_response.status_code == 200:
        push_notification(
            "sams:asset:unlock_asset",
            item_id=asset_id,
            user_id=get_user_id(True),
            session_id=get_auth()["_id"],
            _etag=unlock_asset_response.json()["_etag"],
            extension="sams",
        )
    return unlock_asset_response.json(), unlock_asset_response.status_code


def unlock_asset_by_user(user_id, session_id):
    unlock_asset_response = get_sams_client().assets.unlock_assets_by_user(
        external_user_id=user_id, external_session_id=session_id
    )
    if unlock_asset_response.status_code == 200:
        push_notification(
            "sams:asset:session_unlock", user_id=get_user_id(True), session_id=get_auth()["_id"], extension="sams"
        )
    return unlock_asset_response.status_code


@assets_bp.route("/sams/assets/tags", methods=["GET"])
def get_assets_tags():

    search_query = request.args.to_dict().get("query")
    query = {
        "bool": {
            "must": [{"query_string": {"query": search_query, "default_field": "tags.name", "default_operator": "AND"}}]
        }
    }

    tags_response = get_sams_client().assets.get_tag_codes(query=query if search_query else search_query)
    return tags_response



from typing import Any
from flask_babel import lazy_gettext
import superdesk

from superdesk import get_backend
from apps.highlights.resource import HighlightsResource, MarkedForHighlightsResource
from apps.highlights.service import HighlightsService, MarkedForHighlightsService
from .generate import GenerateHighlightsService, GenerateHighlightsResource


def init_app(app) -> None:
    endpoint_name = "highlights"
    service: Any = HighlightsService(endpoint_name, backend=get_backend())
    HighlightsResource(endpoint_name, app=app, service=service)

    endpoint_name = "marked_for_highlights"
    service = MarkedForHighlightsService(endpoint_name, backend=get_backend())
    MarkedForHighlightsResource(endpoint_name, app=app, service=service)

    endpoint_name = "generate_highlights"
    service = GenerateHighlightsService(endpoint_name, backend=get_backend())
    GenerateHighlightsResource(endpoint_name, app=app, service=service)

    superdesk.privilege(
        name="highlights_read",
        label=lazy_gettext("Highlights - read"),
        description=lazy_gettext("Can see a list of highlights."),
    )
    superdesk.privilege(
        name="highlights",
        label=lazy_gettext("Highlights/Summary List Management"),
        description=lazy_gettext("User can manage highlights/summary lists."),
    )
    superdesk.privilege(
        name="mark_for_highlights",
        label=lazy_gettext("Mark items for Highlights/Summary Lists"),
        description=lazy_gettext("User can mark items for Highlights/Summary Lists."),
    )



import superdesk
from flask_babel import lazy_gettext


def init_app(app) -> None:
    superdesk.register_default_user_preference(
        "monitoring:view",
        {
            "type": "string",
            "allowed": ["list", "swimlane"],
            "view": "list",
            "default": "list",
        },
        label=lazy_gettext("Monitoring view"),
        category=lazy_gettext("monitoring"),
    )

    superdesk.register_default_session_preference("monitoring:view:session", None)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2020 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from io import BytesIO
from datetime import datetime

from werkzeug.wsgi import wrap_file
from flask import request, current_app as app

from superdesk.default_settings import strtobool


class SuperdeskFile(BytesIO):
    _name: str
    filename: str
    content_type: str
    length: int
    upload_date: datetime
    md5: str

    @property
    def name(self):
        return self._name


def generate_response_for_file(
    file: SuperdeskFile,
    cache_for: int = 3600 * 24 * 30,  # 30d cache
    buffer_size: int = 1024 * 256,
    content_disposition: str = None,
):
    data = wrap_file(request.environ, file, buffer_size=buffer_size)
    response = app.response_class(data, mimetype=file.content_type, direct_passthrough=True)
    response.content_length = file.length
    response.last_modified = file.upload_date
    response.set_etag(file.md5)
    response.cache_control.max_age = cache_for
    response.cache_control.s_max_age = cache_for
    response.cache_control.public = True
    response.make_conditional(request)

    if content_disposition:
        response.headers["Content-Disposition"] = content_disposition
    else:
        filename = "; filename={}".format(file.filename or file.name) if file.filename or file.name else ""
        if strtobool(request.args.get("download", "False")):
            response.headers["Content-Disposition"] = "Attachment" + filename
        else:
            response.headers["Content-Disposition"] = "Inline" + filename

    return response



from typing import Optional
from flask import current_app
from eve.flaskapp import Eve
from sams_client import SamsClient

_client: SamsClient = None


def get_sams_client(app: Optional[Eve] = None) -> SamsClient:
    global _client

    if not _client:
        if app is None:
            app = current_app

        _client = SamsClient(
            {
                "HOST": app.config.get("SAMS_HOST"),
                "PORT": app.config.get("SAMS_PORT"),
            }
        )

    return _client



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.services import Service
from superdesk.notification import push_notification

from eve.utils import config
from apps.auth import get_user_id


class SystemMessagesService(Service):
    def on_create(self, docs):
        for doc in docs:
            doc["user_id"] = get_user_id()

    def on_created(self, docs):
        """
        Send notification
        :param docs:
        :return:
        """
        push_notification("system_message:created", _id=[doc.get(config.ID_FIELD) for doc in docs])

    def on_update(self, updates, original):
        updates["user_id"] = get_user_id()

    def on_updated(self, updates, original):
        """
        Send notifification
        :param updates:
        :param original:
        :return:
        """
        push_notification("system_message:updated", _id=[original.get(config.ID_FIELD)])

    def on_deleted(self, doc):
        """
        Send a notification
        :param doc:
        :return:
        """
        push_notification("system_message:deleted", _id=[doc.get(config.ID_FIELD)])



from flask_babel._compat import text_type as text_type
from typing import Any

class LazyString:
    def __init__(self, func: Any, *args: Any, **kwargs: Any) -> None: ...
    def __getattr__(self, attr: Any): ...
    def __len__(self): ...
    def __getitem__(self, key: Any): ...
    def __iter__(self) -> Any: ...
    def __contains__(self, item: Any): ...
    def __add__(self, other: Any): ...
    def __radd__(self, other: Any): ...
    def __mul__(self, other: Any): ...
    def __rmul__(self, other: Any): ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __eq__(self, other: Any) -> Any: ...
    def __ne__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...
    def __html__(self): ...
    def __hash__(self) -> Any: ...
    def __mod__(self, other: Any): ...
    def __rmod__(self, other: Any): ...



from functools import wraps
from flask import request
from flask import current_app as app


def blueprint_auth():
    """
    This decorator is used to add authentication to a Flask Blueprint
    """

    def fdec(f):
        @wraps(f)
        def decorated(*args, **kwargs):
            auth = app.auth
            if not auth.authorized([], "_blueprint", request.method):
                return auth.authenticate()
            return f(*args, **kwargs)

        return decorated

    return fdec
